@@ -90,16 +90,17 @@ struct Opt {
     #[structopt(long)]
     outprefix: PathBuf,
 
-    /// Buffer size in KB for reading chunks of input.
+    /// Buffer size in KB for buffering output before it's sent to
+    /// folders from a mapper.
     ///
-    /// Note memory usage is O(bufsize * nthreads)
+    /// Note memory usage is O(bufsize * nthreads).
     #[structopt(long)]
     bufsize: Option<usize>,
 
     // TODO: consider sort-like KEYDEF -k --key which wouldn't hash if n (numeric) flag set
     /// Print debug information to stderr.
     #[structopt(long)]
-    verbose: Option<bool>,
+    verbose: bool,
 
     // TODO: this isn't very useful as an option, consider removing entirely
     // or allowing a max_mappers and max_folders which controls maximum
@@ -115,15 +116,17 @@ struct Opt {
 
 fn main() {
     let opt = Opt::from_args();
-    let verbose = opt.verbose.unwrap_or(false);
+    let verbose = opt.verbose;
     let nthreads = opt.nthreads.unwrap_or(num_cpus::get_physical());
     let mapper_cmd = opt.mapper.as_deref().unwrap_or("cat");
     let folder_cmd = &opt.folder;
-    let bufsize = opt.bufsize.unwrap_or(16) * 1024;
+    let bufsize = opt.bufsize.unwrap_or(64) * 1024;
     let queuesize = 256;
 
     assert!(!opt.infile.is_empty());
-    // TODO: could play with queuesize and mapper:folder ratio tuning.
+    // TODO: Assume bufsize is fixed due to memory constraints.
+    //
+    // We could play with queuesize and mapper:folder ratio tuning.
     // For map-constrained tasks, reducing folders past 1:1 ratio
     // probably doesn't help since folders sitting idle don't hurt anyone.
     // However, for fold-constrained tasks lower mapper ratios like 1:2, 1:4,
@@ -147,35 +150,32 @@ fn main() {
 
     // Allow enough chunks for parallelism but not so few the chunksize
     // is small.
-    let chunks = fileblocks::chunkify_multiple(&opt.infile, nthreads, bufsize);
+    let read_chunk_size = 16 * 1024;
+    let chunks = fileblocks::chunkify_multiple(&opt.infile, nthreads, read_chunk_size);
     let nthreads = chunks.len(); // smaller b/c of min bufsize
     assert!(nthreads >= 1);
 
-    let mut mapper_processes: Vec<_> = (0..nthreads)
-        .map(|i| {
+    let mut mapper_processes: Vec<_> = chunks
+        .iter()
+        .enumerate()
+        .map(|(i, chunk)| {
             Command::new("/bin/bash")
                 .arg("-c")
-                .arg(mapper_cmd)
-                .stdin(Stdio::piped())
+                .arg(format!(
+                    "/bin/bash -c 'head -c {} | {}'",
+                    chunk.nbytes(),
+                    mapper_cmd
+                ))
+                .stdin(chunk.file())
                 .stdout(Stdio::piped())
                 .spawn()
                 .unwrap_or_else(|err| panic!("error spawn map child {}: {}", i, err))
         })
         .collect();
 
-    let (mapper_inputs, mapper_outputs): (Vec<_>, Vec<_>) = mapper_processes
+    let mapper_outputs: Vec<_> = mapper_processes
         .iter_mut()
-        .map(|child| {
-            let stdin = child.stdin.take().unwrap();
-            let stdout = child.stdout.take().unwrap();
-            (stdin, stdout)
-        })
-        .unzip();
-
-    let mapper_input_threads: Vec<_> = mapper_inputs
-        .into_iter()
-        .zip(chunks.into_iter())
-        .map(|(input, chunk)| thread::spawn(move || chunk.dump(input)))
+        .map(|child| child.stdout.take().unwrap())
         .collect();
 
     let (txs, rxs): (Vec<_>, Vec<_>) = (0..nthreads).map(|_| sync_channel(queuesize)).unzip();
@@ -250,9 +250,6 @@ fn main() {
         })
         .collect();
 
-    mapper_input_threads
-        .into_iter()
-        .for_each(|handle| handle.join().expect("map input join"));
     mapper_processes
         .into_iter()
         .for_each(|mut child| assert!(child.wait().expect("wait").success()));
